import re
import ipaddress
import collections

from .mmap_parser import ipv4, ipv6


def memory_first(bytes_obj, num_lines=0, callback=None, callback2=None):
    try:
        file_length = len(bytes_obj)-1
        startpointer = 0
        endpointer = 0
        found_items = {}
        while num_lines != 0 and endpointer < file_length:
            endpointer = bytes_obj.find(b'\n', startpointer, file_length) + 1
            # have to track pointers to print logs in reverse order

            # end of file
            if endpointer == 0:
                endpointer = file_length+1

            if callback == None:
                num_lines -= 1
            else:
                # last line won't end in \n
                success = callback(bytes_obj[startpointer:endpointer], callback2)
                if success:
                    found_items[startpointer] = endpointer

                num_lines -= success
            startpointer = endpointer
        if callback == None:
            return bytes_obj[:endpointer].decode('utf-8')
        else:
            # sucks that we have to do this but le HIGHLIGHTING
            odered_items = collections.OrderedDict(sorted(found_items.items()))
            result = ""
            for k, v in odered_items.items():
                result += bytes_obj[k:v].decode('utf-8')
            return result.rstrip('\n')

    except Exception as e:
        print(e)

def memory_last(bytes_obj, num_lines=0, callback=None, callback2=None):
    try:
        file_length = len(bytes_obj) - 1
        startpointer = file_length
        endpointer = file_length
        #have to track pointers to print logs in reverse order
        found_items = {}
        # is it safe for startpointer > 0
        while num_lines != 0 and startpointer > 0:
            startpointer = bytes_obj.rfind(b'\n', 0, endpointer)

            if callback == None:
                num_lines -= 1
            else:
                # shift by one to be like first
                success = callback(bytes_obj[startpointer+1:endpointer+1], callback2)
                if success:
                    found_items[startpointer+1] = endpointer+1
                num_lines -= success

            # end of file
            # if startpointer + 1 == 0:
            #     startpointer = 0

            endpointer = startpointer
        if callback == None:
            return bytes_obj[startpointer+1:].decode('utf-8')
        else:
            # sucks that we have to do this but reversing it makes the last item we want to print be first unless we reverse again
            odered_items = collections.OrderedDict(sorted(found_items.items()))
            result = ""
            for k, v in odered_items.items():
                result += bytes_obj[k:v].decode('utf-8')
            return result.rstrip('\n')

    except Exception as e:
        print(e)

def memory_timestamp_all(bytes_obj):
    try:
        ts = re.compile(rb'.*(?:[01]\d|2[0123]):(?:[012345]\d):(?:[012345]\d[^\n]*)')

        matches = ts.findall(bytes_obj)
        print(b'\n'.join(matches).decode('utf-8'))

    except Exception as e:
        raise ValueError(e)

def memory_ipv4_all(bytes_obj):
    return memory_first(bytes_obj, -1, ipv4)

def memory_ipv6_all(bytes_obj):
    return memory_first(bytes_obj, -1, ipv6)


import mmap
import re
import ipaddress
import collections

def first(file_obj, num_lines=0, callback=None, callback2=None):
    try:
        with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_READ) as mmap_obj:
            file_length = len(mmap_obj)-1
            startpointer = 0
            endpointer = 0
            found_items = {}
            while num_lines != 0 and endpointer < file_length:
                endpointer = mmap_obj.find(b'\n', startpointer, file_length) + 1
                # have to track pointers to print logs in reverse order

                # end of file
                if endpointer == 0:
                    endpointer = file_length+1

                if callback == None:
                    num_lines -= 1
                else:
                    # last line won't end in \n
                    success = callback(mmap_obj[startpointer:endpointer], callback2)
                    if success:
                        found_items[startpointer] = endpointer

                        # print(mmap_obj[startpointer:endpointer].decode('utf-8').rstrip('\n'))
                    num_lines -= success
                startpointer = endpointer
            if callback == None:
                return mmap_obj[:endpointer].decode('utf-8')
            else:
                # sucks that we have to do this but le HIGHLIGHTING
                odered_items = collections.OrderedDict(sorted(found_items.items()))
                result = ""
                for k, v in odered_items.items():
                    result += mmap_obj[k:v].decode('utf-8')
                return result.rstrip('\n')

    except Exception as e:
        print(e)

def last(file_obj, num_lines=0, callback=None, callback2=None):
    try:
        with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_READ) as mmap_obj:
            file_length = len(mmap_obj) - 1
            startpointer = file_length
            endpointer = file_length
            #have to track pointers to print logs in reverse order
            found_items = {}
            # is it safe for startpointer > 0
            while num_lines != 0 and startpointer > 0:
                startpointer = mmap_obj.rfind(b'\n', 0, endpointer)

                if callback == None:
                    num_lines -= 1
                else:
                    # shift by one to be like first
                    success = callback(mmap_obj[startpointer+1:endpointer+1], callback2)
                    if success:
                        found_items[startpointer+1] = endpointer+1
                    num_lines -= success

                # end of file
                # if startpointer + 1 == 0:
                #     startpointer = 0

                endpointer = startpointer
            if callback == None:
                return mmap_obj[startpointer+1:].decode('utf-8')
            else:
                # sucks that we have to do this but reversing it makes the last item we want to print be first unless we reverse again
                odered_items = collections.OrderedDict(sorted(found_items.items()))
                result = ""
                for k, v in odered_items.items():
                    result += mmap_obj[k:v].decode('utf-8')
                return result.rstrip('\n')

    except Exception as e:
        print(e)

def timestamp(line, callback=None):
    if line:
        pattern = re.compile(rb'(?:[01]\d|2[0123]):(?:[012345]\d):(?:[012345]\d)')

        if len(pattern.findall(line)):

            if callback == None:
                # found timestamp
                return 1
            else:
                return callback(line)

    # timestamp not found
    return 0

def ipv4(line, callback=None):

    if line:
        pattern = re.compile(rb'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})')
        ips = pattern.findall(line)

        if len(ips):
            for ip in ips:
                try:
                    ipaddress.IPv4Address(ip.decode('utf-8'))

                    # found ipv4
                    return 1
                except Exception as e:
                    pass

    # ipv4 not found
    return 0

def ipv6(line, callback=None):

    if line:
        pattern = rb'^.*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?'

        if re.match(pattern, line):

            # found ipv6
            return 1

    # ipv6 not found
    return 0


def timestamp_all(file_obj):
    try:
        ts = re.compile(rb'.*(?:[01]\d|2[0123]):(?:[012345]\d):(?:[012345]\d[^\n]*)')

        with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_READ) as mmap_obj:
            # prepend \n to begining and search for \n timestamp \n, concat together and print
            matches = ts.findall(mmap_obj)
            print(b'\n'.join(matches).decode('utf-8'))

    except Exception as e:
        raise ValueError(e)

def ipv4_all(file_obj):
    return first(file_obj, -1, ipv4)

    # ip_pattern = re.compile(rb'(.*\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}[^\n]*)')
    #
    # try:
    #     with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_READ) as mmap_obj:
    #         # prepend \n to begining and search for \n timestamp \n, concat together and print
    #         matches = ip_pattern.findall(mmap_obj)
    #         iter = 0
    #         while iter < len(matches):
    #             valid_pattern = re.compile(rb'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})')
    #             ips = valid_pattern.findall(matches[iter])
    #             for ip in ips:
    #                 try:
    #                     ipaddress.IPv4Address(ip.decode('utf-8'))
    #                     print(matches[iter].decode('utf-8'))
    #                     iter += 1
    #                     break
    #                 except Exception as e:
    #                     print(e)
    #             iter += 1
    # except Exception as e:
    #     print(e)

def ipv6_all(file_obj):
    # use ipv6 callback for every line in file, by using -1 the first loop will run until the end
    return first(file_obj, -1, ipv6)

def ipv4_ipv6_all(line, callback=None):
    if line:
        ipv4_pattern = re.compile(rb'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})')
        ipv6_pattern = rb'^.*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?'

        ipv4_ips = ipv4_pattern.findall(line)

        if len(ipv4_ips):
            for ip in ipv4_ips:
                try:
                    ipaddress.IPv4Address(ip.decode('utf-8'))

                    # found ipv4
                    return 1
                except Exception as e:
                    pass


        if re.match(ipv6_pattern, line):

            # found ipv6
            return 1

    # ipv6 not found
    # ipv4 not found
    return 0


#!/usr/bin/python3

import argparse
import mmap
import sys
import re
import ipaddress
from collections import namedtuple

from .cmd_action import *
from .argparser import *


def main():



    terminalargs = sys.argv[1:]

    if '-h' in terminalargs or '--help' in terminalargs:
        print(help)
        sys.exit(1)
    len_termargs = len(terminalargs)
    if len_termargs == 0:
        print(help)
        sys.exit(1)

    try:
        with open(sys.argv[-1], 'rb') as file_obj:
            mmap_possible = 1
            args = argparser(terminalargs, mmap_possible)
            cmd_action(file_obj, args, mmap_possible)
    except Exception as e:
        try:
            if not sys.stdin.isatty():
                file_obj = sys.stdin.buffer.read()
                mmap_possible = 0
                args = argparser(terminalargs, mmap_possible)
                cmd_action(file_obj, args, mmap_possible)

        except Exception as e:
            print("Error reading log file please pipe data to python file or have log file at end of command\n")
            print(help)
            sys.exit(1)

    # now the fun part



if __name__ == '__main__':

    main()


import ipaddress
import re

def validate_first(input):
    try:
        i = int(input)
    except Exception as e:
        return False

    if int(i) <= 0:
        return False
    return True

def validate_last(input):
    try:
        i = int(input)
    except Exception as e:
        return False

    if int(i) <= 0:
        return False
    return True

def validate_timestamp(input):
    if input != -1:
        # Regex to check valid time in 24-hour format.
        pattern = rb'(?:[01]\d|2[0123]):(?:[012345]\d):(?:[012345]\d)'

        # Compile the ReGex
        p = re.compile(pattern)
        m = re.search(p, input)
        # matched the ReGex otherwise False
        if m is None:
            raise ValueError("Input timestamp as HH:MM:SS")
    return True


def validate_ipv4(input):
    if input != -1:
        try:
            ipaddress.IPv4Address(input)
        except Exception as e:
            return False
    return True

def validate_ipv6(input):
    if input != -1:
        try:
            ipaddress.IPv6Address(input)
        except Exception as e:
            return False
    return True


import re
import random

from .validator import *
from .mmap_parser import *
from .memory_parser import  *

def lighter(line):
    """
    prints table of formatted text format options
    """
    # fg = random.randint(30, 38)
    # just make text black
    fg = 30
    s1 = ''
    # bg = random.randint(40, 48)
    # don't want a black backgroun
    bg = random.randint(41,48)
    if bg == fg:
        if fg < 35:
            bg = bg in range(45, 48)
        else:
            bg = bg in range(40, 44)

    format = ';'.join([str(1), str(fg), str(bg)])
    s1 += '\x1b[%sm %s \x1b[0m' % (format, line)
    return s1

def highlighter_ipv4(results):
    pattern = re.compile(r'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})')
    ipv4s = pattern.findall(results)
    for ip in set(ipv4s):
        results = results.replace(ip, lighter(ip))
    return results

def highlighter_ipv6(results):
    ipv6_address = re.compile(
        '(?:(?:[0-9A-Fa-f]{1,4}:){6}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|::(?:[0-9A-Fa-f]{1,4}:){5}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){4}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){3}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,2}[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){2}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,3}[0-9A-Fa-f]{1,4})?::[0-9A-Fa-f]{1,4}:(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,4}[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,5}[0-9A-Fa-f]{1,4})?::[0-9A-Fa-f]{1,4}|(?:(?:[0-9A-Fa-f]{1,4}:){,6}[0-9A-Fa-f]{1,4})?::)')

    ipv6s = ipv6_address.findall(results)
    for ip in set(ipv6s):
        results = results.replace(ip, lighter(ip))
    return results

def highlight_both(results):
    ipv6_address = re.compile(
        '(?:(?:[0-9A-Fa-f]{1,4}:){6}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|::(?:[0-9A-Fa-f]{1,4}:){5}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){4}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){3}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,2}[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:){2}(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,3}[0-9A-Fa-f]{1,4})?::[0-9A-Fa-f]{1,4}:(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,4}[0-9A-Fa-f]{1,4})?::(?:[0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}|(?:(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}(?:[0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|(?:(?:[0-9A-Fa-f]{1,4}:){,5}[0-9A-Fa-f]{1,4})?::[0-9A-Fa-f]{1,4}|(?:(?:[0-9A-Fa-f]{1,4}:){,6}[0-9A-Fa-f]{1,4})?::)')

    ipv6s = ipv6_address.findall(results)
    for ip in set(ipv6s):
        results = results.replace(ip, lighter(ip))

    pattern = re.compile(r'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})')
    ipv4s = pattern.findall(results)
    for ip in set(ipv4s):
        results = results.replace(ip, lighter(ip))

    return results

def cmd_action(file_object, args, mmap_possible):
    try:
        if args.first:
            if args.timestamps:
                if args.ipv4 and args.ipv6:
                    if mmap_possible:
                        print(highlight_both(first(file_object, args.first, timestamp, ipv4_ipv6_all)))
                    else:
                        print(highlight_both(memory_first(file_object, args.first, timestamp, ipv4_ipv6_all)))
                elif args.ipv4:
                    if mmap_possible:
                        print(highlighter_ipv4(first(file_object, args.first, timestamp, ipv4)))
                    else:
                        print(highlighter_ipv4(memory_first(file_object, args.first, timestamp, ipv4)))
                elif args.ipv6:
                    if mmap_possible:
                        print(highlighter_ipv6(first(file_object, args.first, timestamp, ipv6)))
                    else:
                        print(highlighter_ipv6(memory_first(file_object, args.first, timestamp, ipv6)))
                else:
                    if mmap_possible:
                        print(first(file_object, args.first, timestamp))
                    else:
                        print(memory_first(file_object, args.first, timestamp))
            elif args.ipv4 and args.ipv6:
                if mmap_possible:
                    print(highlight_both(first(file_object, args.first, ipv4_ipv6_all)))
                else:
                    print(highlight_both(memory_first(file_object, args.first, ipv4_ipv6_all)))
            elif args.ipv4:
                if mmap_possible:
                    print(highlighter_ipv4(first(file_object, args.first, ipv4)))
                else:
                    print(highlighter_ipv4(memory_first(file_object, args.first, ipv4)))
            elif args.ipv6:
                if mmap_possible:
                    print(highlighter_ipv6(first(file_object, args.first, ipv6)))
                else:
                    print(highlighter_ipv6(memory_first(file_object, args.first, ipv6)))

            else:
                if mmap_possible:
                    print(first(file_object, args.first))
                else:
                    print(memory_first(file_object, args.first))

        elif args.last:
            if args.timestamps:
                if args.ipv4 and args.ipv6:
                    if mmap_possible:
                        print(highlight_both(last(file_object, args.last, timestamp, ipv4_ipv6_all)))
                    else:
                        print(highlight_both(memory_last(file_object, args.last, timestamp, ipv4_ipv6_all)))
                elif args.ipv4:
                    if mmap_possible:
                        print(highlighter_ipv4(last(file_object, args.last, timestamp, ipv4)))
                    else:
                        print(highlighter_ipv4(memory_last(file_object, args.last, timestamp, ipv4)))
                elif args.ipv6:
                    if mmap_possible:
                        print(highlighter_ipv6(last(file_object, args.last, timestamp, ipv6)))
                    else:
                        print(highlighter_ipv6(memory_last(file_object, args.last, timestamp, ipv6)))
                else:
                    if mmap_possible:
                        print(last(file_object, args.last, timestamp))
                    else:
                        print(memory_last(file_object, args.last, timestamp))
            elif args.ipv4 and args.ipv6:
                if mmap_possible:
                    print(highlight_both(last(file_object, args.last, ipv4_ipv6_all)))
                else:
                    print(highlight_both(memory_last(file_object, args.last, ipv4_ipv6_all)))
            elif args.ipv4:
                if mmap_possible:
                    print(highlighter_ipv4(last(file_object, args.last, ipv4)))
                else:
                    print(highlighter_ipv4(memory_last(file_object, args.last, ipv4)))
            elif args.ipv6:
                if mmap_possible:
                    print(highlighter_ipv6(last(file_object, args.last, ipv6)))
                else:
                    print(highlighter_ipv6(memory_last(file_object, args.last, ipv6)))

            else:
                if mmap_possible:
                    print(last(file_object, args.last))
                else:
                    print(memory_last(file_object, args.last))

        elif args.timestamps:
            if args.ipv4 and args.ipv6:
                if mmap_possible:
                    print(first(file_object, -1, timestamp, ipv4_ipv6_all))
                else:
                    print(memory_first(file_object, -1, timestamp, ipv4_ipv6_all))
            elif args.ipv4:
                if mmap_possible:
                    print(highlighter_ipv4(first(file_object, -1, timestamp, ipv4)))
                else:
                    print(highlighter_ipv4(memory_first(file_object, -1, timestamp, ipv4)))
            elif args.ipv6:
                if mmap_possible:
                    print(highlighter_ipv6(first(file_object, -1, timestamp, ipv6)))
                else:
                    print(highlighter_ipv6(memory_first(file_object, -1, timestamp, ipv6)))
            else:
                timestamp_all(file_object)
                if mmap_possible:
                    timestamp_all(file_object)
                else:
                    memory_timestamp_all(file_object)

        elif args.ipv4:
            if args.ipv6:
                print(highlight_both(first(file_object, -1, ipv4_ipv6_all)))
                if mmap_possible:
                    print(highlight_both(first(file_object, -1, ipv4_ipv6_all)))
                else:
                    print(highlight_both(memory_first(file_object, -1, ipv4_ipv6_all)))
            else:
                if mmap_possible:
                    print(highlighter_ipv4(ipv4_all(file_object)))
                else:
                    print(highlighter_ipv4(memory_ipv4_all(file_object)))
        elif args.ipv6:
            print(highlighter_ipv6(ipv6_all(file_object)))
            if mmap_possible:
                print(highlighter_ipv6(ipv6_all(file_object)))
            else:
                print(highlighter_ipv6(memory_ipv6_all(file_object)))

        else:
            raise ValueError("Error parsing cmdline options")

    except Exception as e:
        ValueError(e)


import sys
import re
import ipaddress
from collections import namedtuple

from .validator import *

help = '''
usage: ./util.py [OPTION]... [FILE]
usage: cat [FILE] | ./util.py [OPTION]
usage: mmap-logparser [OPTION]... [FILE]
usage: cat [FILE] | mmap-logparser[OPTION]

Parse logs of various kinds

optional arguments:
  -h, --help            show this help message and exit
  -f, --first NUM       Print first NUM lines
  -l, --last NUM        Print last NUM lines
  -t, --timestamps      Print lines that contain a timestamp in HH:MM:SS format
  -i, --ipv4 IP         Print lines that contain an IPv4 address, matching IPs (Optional) highlighted
  -I, --ipv6 IP         Print lines that contain an IPv6 address, matching IPs (Optional) highlighted

'''

def argparser(terminalargs, mmap_possible):
    # ugggh wish i could use argparser for this
    # hacky way

    args = {}

    def argcheck(key):
        if key in args.keys():
            print("Please only enter command once\n")
            print(help)
            sys.exit(1)



    len_termargs = len(terminalargs)
    if len_termargs == 0:
        print(help)
        sys.exit(1)



    if mmap_possible:
        len_termargs -= 1
        terminalargs = terminalargs[:len_termargs]

    if len_termargs == 0:
        print("Please input commands\n")
        print(help)
        sys.exit(1)
    else:
        i = 0
        while i < len_termargs:
            if terminalargs[i] == '--first' or terminalargs[i] == '-f':
                argcheck(terminalargs[i])
                if i +1 >= len_termargs or not validate_first(terminalargs[i+1]):
                    print("Input a positive number for the --first command\n")
                    print(help)
                    sys.exit(1)
                args['first'] = int(terminalargs[i+1])
                i += 2
            elif terminalargs[i] == '--last' or terminalargs[i] == '-l':
                argcheck(terminalargs[i])
                if i +1 > len_termargs or not validate_last(terminalargs[i+1]):
                    print("Input a positive number for the --last command\n")
                    print(help)
                    sys.exit(1)
                args['last'] = int(terminalargs[i+1])
                i += 2
            elif terminalargs[i] == '--timestamps' or terminalargs[i] == '-t':
                argcheck(terminalargs[i] )
                args['timestamps'] = "-1"
                i+=1

            elif terminalargs[i] == '--ipv4' or terminalargs[i] == '-i':
                argcheck(terminalargs[i] )
                invalids = ['--first', '-f','--last', 'l', '--timestamps', '-t', '-i', '--ipv4', '-I', '--ipv6']
                if i +1 < len_termargs and (terminalargs[i+1] not in invalids):
                    if not validate_ipv4(terminalargs[i+1]):
                        print("Input a valid ipv4 address for the --ipv4 command\n")
                        print(help)
                        sys.exit(1)
                    args['ipv4'] = terminalargs[i+1]
                    i += 2

                else:
                    args['ipv4'] = "-1"
                    i += 1

            elif terminalargs[i] == '--ipv6' or terminalargs[i] == '-I':
                argcheck(terminalargs[i] )
                invalids = ['--first', '-f','--last', 'l', '--timestamps', '-t', '-i', '--ipv4', '-I', '--ipv6']
                if i +1 < len_termargs and (terminalargs[i+1] not in invalids):
                    if not validate_ipv6(terminalargs[i+1]):
                        print("Input a valid ipv6 address for the --ipv6 command\n")
                        print(help)
                        sys.exit(1)
                    args['ipv6'] = terminalargs[i+1]
                    i += 2
                else:
                    args['ipv6'] = "-1"
                    i += 1

            else:
                print("You inputted an unknown commands\n")
                print(help)
                sys.exit(1)

    # now the fun part
    # later refactor to only have arg be object at beginning
    # uggh this sucks
    try:
        val = args['first']
    except KeyError:
        args['first'] = None

    try:
        val = args['last']
    except KeyError:
        args['last'] = None

    try:
        val = args['timestamps']
    except KeyError:
        args['timestamps'] = None

    try:
        val = args['ipv4']
    except KeyError:
        args['ipv4'] = None

    try:
        val = args['ipv6']
    except KeyError:
        args['ipv6'] = None
    args = namedtuple("args", args.keys())(*args.values())

    return args


from setuptools import setup, find_packages

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="redparser",
    version="1.0.1",
    author="mtclinton",
    author_email="max@mtclinton.com",
    description='Simple Python library to use parse log files',
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    packages=find_packages(exclude=("tests",)),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.5',
    entry_points={
        "console_scripts": [
            "redparser=redparser.__main__:main",
        ]
    },
)